<?php

class Tvg_Service {

    protected $_headers;
    protected $_apiUrl = "https://appgw.eimskip.net/gateway/TVGXpressOperations/1.0/";
    protected $_response;
    protected $_client;
    protected $_service = "TVGXpress";
    
    public function __construct($username,$apiKey,$test=false) {
        $this->_headers = array(
            'Content-Type'=>'application/json',
            'Accept'=>'application/json',
            'Authorization'=>sprintf('Basic %s', base64_encode($username.':'.$apiKey))
        );
        
        
        if($test){
            $this->_apiUrl = "https://appgw-test.eimskip.net/gateway/TVGXpressOperations/1.0/";
        }

        $this->_client = new WP_Http();
    }
    
    

    /**
     * getLabels
     * 
     * This service is used to get labels for a given shipment. A label contains details about the shipment it
     * refers to. A ZPL string or a PDF file can be requested.
     * 
     * @param string $shipmentId - The shipment number for which to get labels.
     * @param array $params [labelType] - Enum: "ZPL_STRING", "PDF".
     * @return array [status] - Segment that indicates whether the operation was successful or not.
        
        [response/label] - A string containing the label data.
            If PDF was the requested label type, the file is
            base64 encoded. If a ZPL string was requested, a
            ZPL string is returned.

     */
    public function getLabels($shipmentId,$params=array()){
        return $this->get(sprintf('labels/%s',$shipmentId),$params);
    }
    
    /**
     * getShipment
     * Returns detailed information about the requested shipment.
     * 
     * @param string $shipmentId 
     * Reference number of the original shipment.
     * Received as a response to createShipment method
     * @return array [status] Segment that indicates whether the operation was successful or not.<br><br> 
     *   [response] Segment describing the shipment to be created.<br>
     *       -[shipment]<br>
     *           --[Sender] type Party<br>
     *           --[Receiver] type Party<br>
     *           --[Parcel] type Parcel<br>
     *           --[Barcode] The barcode for the created shipment. If left empty, a barcode will be automatically generated.<br>
     */
    public function getShipment($shipmentId){
        return $this->get(sprintf('shipments/info/%s',$shipmentId));
    }
    
    
    /**
     * getShipment
     * Returns detailed information about all shipments created within the specified time interval. 
     * Maximum date span is 30 days.
     * 
     * @param date $dateFrom Pattern: yyyy-MM-dd HH:mm:ss
     * @param date $dateTo Pattern: yyyy-MM-dd HH:mm:ss
     * 
     * @return array [status] Segment that indicates whether the operation was successful or not.<br><br> 
     *   [response] Segment describing the shipment to be created.<br>
     *       -[shipment]<br>
     *           --[Sender] type Party<br>
     *           --[Receiver] type Party<br>
     *           --[Parcel] type Parcel<br>
     *           --[Barcode] The barcode for the created shipment. If left empty, a barcode will be automatically generated.<br>
     */
    public function getShipments($dateFrom,$dateTo){
        return $this->get('shipments/info',['dateFrom'=>$dateFrom,'dateTo'=>$dateTo]);
    }
    
    /**
     * getShipmentStatus
     * Returns key information about the requested shipment.
     * @param string $shipmentId Reference number of the original shipment.
     * @return array [status] Segment that indicates whether the operation was successful or not.<br><br> 
     *   [response] Segment describing the shipment to be created.<br>
     *      -[Signature] <i>Signature.</i><br>
     *      -[LastChanged] <i>Time of last change. Empty if no changes have been made.</i><br>
     *      -[ReceivedDate] <i>Time received. Empty if nothing has been received.</i><br>
     *      -[DeliveryRequestDate] <i>Time of delivery request. Empty if delivery request has not been made.</i><br>
     *      -[InShelfDate] <i>Time when put in shelf. Empty if shipment has not been put in shelf.</i><br>
     *      -[TrackingNo] <i>Tracking number.</i>
     */
    public function getShipmentStatus($shipmentId){
        return $this->get(sprintf('shipments/status/%s',$shipmentId));
    }

    public function getShipmentStatusHtml($shipmentId){
        $response = $this->get(sprintf('shipments/status/%s',$shipmentId));

        $html = '';

        if(!empty($response) && $response->status->code == '200'){
            $statusIndex = 1;
            $statusIndex = (strlen($response->response->shipmentStatus->ReceivedDate) > 0)?2:$statusIndex;
            $statusIndex = (strlen($response->response->shipmentStatus->DeliveryRequestDate) > 0)?3:$statusIndex;
            $statusIndex = (strlen($response->response->shipmentStatus->DeliveryConfirmedDate) > 0)?4:$statusIndex;

            $html = '<ul class="tvg-shipping-status">
                <li class="'.$this->getStatusClass(1,$statusIndex).'">Pakki skráður</li>
                <li class="'.$this->getStatusClass(2,$statusIndex).'">Pakkinn kominn til flutningsaðila</li>
                <li class="'.$this->getStatusClass(3,$statusIndex).'">Pakkinn er á leiðinni til viðskiptavinar</li>
                <li class="'.$this->getStatusClass(4,$statusIndex).'">Pakki hefur komist til skila</li>
                </ul>';
        }
        return $html;
    }

    private function getStatusClass($index, $statusIndex): string
    {
        if($index == $statusIndex) return 'active';
        else if($index > $statusIndex) return 'disabled';
        else return 'done';
    }
    
    
    /**
     * createShipment
     * This service is used to register a new shipment in the TVGXpress system. If registration is successful,
     * the service returns a shipment number generated for the request.
     * @param array $params [request]
     * -[shipment] <br>
     * --[Sender] type Party<br>
     * --[Receiver] type Party<br>
     * --[Parcel] type Parcel<br>
     * 
     * @return array [status] Segment that indicates whether the operation was
     * successful or not.<br>
     * [response] shipmentNumber A unique id for the relevant transmission.
     */
    public function createShipment($params){
        return $this->post('shipments',$params);
    }
            
    protected function get($url, $params = null) {
        $data = array(
            'headers' => $this->_headers
        );
        if(!empty($params)){
            $url = $url.'?'.http_build_query($params);
        }
        
        try{
            $response = wp_remote_get($this->_apiUrl.$url,$data);
            $code = wp_remote_retrieve_response_code($response);
            $retResponse = json_decode(wp_remote_retrieve_body( $response ));
            $this->log(
                wc_print_r(
                    array(
                        'code' => $code,
                        'url' => $this->_apiUrl.$url,
                        'data'=> $params,
                        'response' => $retResponse
                    ), true
                )
            );
           return $retResponse;
        }catch(exception $e){
            $this->log(
                wc_print_r(
                    array(
                        'code' => $e->getCode(),
                        'url' => $this->_apiUrl.$url,
                        'data'=> $params,
                        'response' => $e->getMessage()
                    ), true
                )
            );
        }
    }
    
    protected function post($url,$params) {

        $data = array(
            'headers' => $this->_headers,
            'body'=>wp_json_encode($params),
            'data_format' => 'body'
        );

        try{
            $response = wp_remote_post($this->_apiUrl.$url,$data);
            $code = wp_remote_retrieve_response_code($response);
            $retResponse = json_decode(wp_remote_retrieve_body( $response ));
            $this->log(
                wc_print_r(
                    array(
                        'code' => $code,
                        'url' => $this->_apiUrl.$url,
                        'data'=> $params,
                        'rsponse' => $retResponse
                    ), true
                )
            );
            return $retResponse;

        }catch(exception $e){
            $this->log(
                wc_print_r(
                    array(
                        'code' => $e->getCode(),
                        'url' => $this->_apiUrl.$url,
                        'data'=> $params,
                        'rsponse' => $e->getMessage()
                    ), true
                )
            );
        }
    }

    public function track_package($body=array()){

        $url = 'https://tvg-shopify-jacd3.ondigitalocean.app/api/collection/track';

        $args = array(
            'timeout' => 0,
            'method' => 'POST',
            'headers' => array(
                'Content-Type' => 'application/json',
                'Connection' => 'keep-alive'
            )
        );

        if(!empty($body)){
            $body = wp_json_encode( $body );
            $args['body'] = $body;
        }
        try {
            $response = wp_remote_request($url, $args);
            $code = wp_remote_retrieve_response_code($response);
            $retResponse = json_decode(wp_remote_retrieve_body( $response ));
            $this->log(
                wc_print_r(
                    array(
                        'code' => $code,
                        'url' => $url,
                        'data'=> json_decode($body),
                        'response' => $retResponse
                    ), true
                )
            );
            return $retResponse;
        }catch(exception $e){
            $this->log(
                wc_print_r(
                    array(
                        'code' => $e->getCode(),
                        'url' => $url,
                        'data'=> json_decode($body),
                        'response' => $e->getMessage()
                    ), true
                )
            );
        }
    }
    
    /**
     * Retrieve last response
     * @return GuzzleHttp\Psr7\Response
     */
    public function getResponse(){
        return $this->_response;
    }

    protected function log($message){
        if (empty($this->log)) {
            $this->log = wc_get_logger();
        }

        $this->log->debug($message, array('source' => 'tvg-xpress'));

    }
}
?>
